<?php

namespace Laraecart\Filter\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Laraecart\Filter\Http\Requests\FilterRequest;
use Laraecart\Filter\Interfaces\FilterRepositoryInterface;
use Laraecart\Filter\Models\Filter;

/**
 * Resource controller class for filter.
 */
class FilterResourceController extends BaseController
{

    /**
     * Initialize filter resource controller.
     *
     * @param type FilterRepositoryInterface $filter
     *
     * @return null
     */
    public function __construct(FilterRepositoryInterface $filter)
    {
        parent::__construct();
        $this->repository = $filter;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Laraecart\Filter\Repositories\Criteria\FilterResourceCriteria::class);
    }

    /**
     * Display a list of filter.
     *
     * @return Response
     */
    public function index(FilterRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Laraecart\Filter\Repositories\Presenter\FilterPresenter::class)
                ->$function();
        }

        $filters = $this->repository->paginate();

        return $this->response->title(trans('filter::filter.names'))
            ->view('filter::filter.index', true)
            ->data(compact('filters'))
            ->output();
    }

    /**
     * Display filter.
     *
     * @param Request $request
     * @param Model   $filter
     *
     * @return Response
     */
    public function show(FilterRequest $request, Filter $filter)
    {

        if ($filter->exists) {
            $view = 'filter::filter.show';
        } else {
            $view = 'filter::filter.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('filter::filter.name'))
            ->data(compact('filter'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new filter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FilterRequest $request)
    {

        $filter = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('filter::filter.name')) 
            ->view('filter::filter.create', true) 
            ->data(compact('filter'))
            ->output();
    }

    /**
     * Create new filter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FilterRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $filter                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('filter::filter.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('filter/filter/' . $filter->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/filter/filter'))
                ->redirect();
        }

    }

    /**
     * Show filter for editing.
     *
     * @param Request $request
     * @param Model   $filter
     *
     * @return Response
     */
    public function edit(FilterRequest $request, Filter $filter)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('filter::filter.name'))
            ->view('filter::filter.edit', true)
            ->data(compact('filter'))
            ->output();
    }

    /**
     * Update the filter.
     *
     * @param Request $request
     * @param Model   $filter
     *
     * @return Response
     */
    public function update(FilterRequest $request, Filter $filter)
    {
        try {
            $attributes = $request->all();

            $filter->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('filter::filter.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('filter/filter/' . $filter->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('filter/filter/' . $filter->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the filter.
     *
     * @param Model   $filter
     *
     * @return Response
     */
    public function destroy(FilterRequest $request, Filter $filter)
    {
        try {

            $filter->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('filter::filter.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('filter/filter/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('filter/filter/' . $filter->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple filter.
     *
     * @param Model   $filter
     *
     * @return Response
     */
    public function delete(FilterRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('filter::filter.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('filter/filter'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/filter/filter'))
                ->redirect();
        }

    }

    /**
     * Restore deleted filters.
     *
     * @param Model   $filter
     *
     * @return Response
     */
    public function restore(FilterRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('filter::filter.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/filter/filter'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/filter/filter/'))
                ->redirect();
        }

    }

}
